@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2021 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,8 +26,10 @@ from tensorflow_datasets.core.utils import type_utils
 SRC_BASE_URL = 'https://github.com/tensorflow/datasets/tree/master/'
 
 # Directory where to store processed datasets.
-DATA_DIR = os.environ.get('TFDS_DATA_DIR',
-                          os.path.join('~', 'tensorflow_datasets'))
+# If modifying this, should also update `scripts/cli/build.py` `--data_dir`
+DATA_DIR = os.environ.get(
+    'TFDS_DATA_DIR', os.path.join('~', 'tensorflow_datasets')
+)
 
 # Suffix of files / directories which aren't finished downloading / extracting.
 INCOMPLETE_SUFFIX = '.incomplete'
@@ -53,6 +55,9 @@ def add_data_dir(data_dir):
   Args:
     data_dir: New data_dir to register.
   """
+  # Remove trailing / to avoid same directory being included twice in the set
+  # with and without a final slash.
+  data_dir = data_dir.rstrip('/')
   _registered_data_dir.add(data_dir)
 
 
