@@ -35,6 +35,7 @@ import json
 import os
 import posixpath
 import tempfile
+from typing import Optional
 
 from absl import logging
 import six
@@ -56,6 +57,7 @@ from google.protobuf import json_format
 # Name of the file to output the DatasetInfo protobuf object.
 DATASET_INFO_FILENAME = "dataset_info.json"
 LICENSE_FILENAME = "LICENSE"
+METADATA_FILENAME = "metadata.json"
 
 
 # TODO(tfds): Do we require to warn the user about the peak memory used while
@@ -78,9 +80,11 @@ class DatasetInfo(object):
                description=None,
                features=None,
                supervised_keys=None,
+               disable_shuffling: bool = False,
                homepage=None,
                citation=None,
                metadata=None,
+               license: Optional[str] = None,  # pylint: disable=redefined-builtin
                redistribution_info=None):
     """Constructs DatasetInfo.
 
@@ -96,11 +100,13 @@ class DatasetInfo(object):
         `info.features`. When calling `tfds.core.DatasetBuilder.as_dataset()`
         with `as_supervised=True`, the `tf.data.Dataset` object will yield
         the (input, target) defined here.
+      disable_shuffling: `bool`, specify whether to shuffle the examples.
       homepage: `str`, optional, the homepage for this dataset.
       citation: `str`, optional, the citation to use for this dataset.
       metadata: `tfds.core.Metadata`, additonal object which will be
         stored/restored with the dataset. This allows for storing additional
         information with the dataset.
+      license: Optional license of the dataset
       redistribution_info: `dict`, optional, information needed for
         redistribution, as specified in `dataset_info_pb2.RedistributionInfo`.
         The content of the `license` subfield will automatically be written to a
@@ -119,12 +125,13 @@ class DatasetInfo(object):
         name=builder.name,
         description=utils.dedent(description),
         version=str(builder.version),
+        disable_shuffling=disable_shuffling,
         config_name=config_name,
         config_description=config_description,
         citation=utils.dedent(citation),
         module_name=str(builder.__module__),
         redistribution_info=dataset_info_pb2.RedistributionInfo(
-            license=utils.dedent(redistribution_info.pop("license")),
+            license=utils.dedent(license or redistribution_info.pop("license")),
             **redistribution_info) if redistribution_info else None)
 
     if homepage:
@@ -177,6 +184,10 @@ class DatasetInfo(object):
   def version(self):
     return self._builder.version
 
+  @property
+  def disable_shuffling(self) -> bool:
+    return self.as_proto.disable_shuffling
+
   @property
   def homepage(self):
     urls = self.as_proto.location.urls
@@ -337,7 +348,7 @@ class DatasetInfo(object):
       f.write(self.as_json)
 
   def read_from_directory(self, dataset_info_dir):
-    """Update DatasetInfo from the JSON file in `dataset_info_dir`.
+    """Update DatasetInfo from the JSON files in `dataset_info_dir`.
 
     This function updates all the dynamically generated fields (num_examples,
     hash, time of creation,...) of the DatasetInfo.
@@ -349,7 +360,7 @@ class DatasetInfo(object):
         should be the root directory of a specific dataset version.
 
     Raises:
-      FileNotFoundError: If the file can't be found.
+      FileNotFoundError: If the dataset_info.json can't be found.
     """
     logging.info("Load dataset info from %s", dataset_info_dir)
 
@@ -377,7 +388,15 @@ class DatasetInfo(object):
       self._features = feature_lib.FeatureConnector.from_config(
           dataset_info_dir
       )
-    if self.metadata is not None:
+    # Restore the MetaDataDict from metadata.json if there is any
+    if (
+        self.metadata is not None
+        or tf.io.gfile.exists(_metadata_filepath(dataset_info_dir))
+    ):
+      # If the dataset was loaded from file, self.metadata will be `None`, so
+      # we create a MetadataDict first.
+      if self.metadata is None:
+        self._metadata = MetadataDict()
       self.metadata.load_metadata(dataset_info_dir)
 
     # Update fields which are not defined in the code. This means that
@@ -465,6 +484,7 @@ class DatasetInfo(object):
         ("dataset_size", self.dataset_size),
         ("features", _indent(repr(self.features))),
         ("supervised_keys", self.supervised_keys),
+        ("disable_shuffling", self.disable_shuffling),
         ("splits", splits),
         ("citation", _indent(f'"""{self.citation}"""')),
         # Proto add a \n that we strip.
@@ -582,24 +602,25 @@ class Metadata(dict):
     raise NotImplementedError()
 
 
+def _metadata_filepath(data_dir):
+  return os.path.join(data_dir, METADATA_FILENAME)
+
+
 class MetadataDict(Metadata, dict):
   """A `tfds.core.Metadata` object that acts as a `dict`.
 
   By default, the metadata will be serialized as JSON.
   """
 
-  def _build_filepath(self, data_dir):
-    return os.path.join(data_dir, "metadata.json")
-
   def save_metadata(self, data_dir):
     """Save the metadata."""
-    with tf.io.gfile.GFile(self._build_filepath(data_dir), "w") as f:
+    with tf.io.gfile.GFile(_metadata_filepath(data_dir), "w") as f:
       json.dump(self, f)
 
   def load_metadata(self, data_dir):
     """Restore the metadata."""
     self.clear()
-    with tf.io.gfile.GFile(self._build_filepath(data_dir), "r") as f:
+    with tf.io.gfile.GFile(_metadata_filepath(data_dir), "r") as f:
       self.update(json.load(f))
 
 
