@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2021 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,7 +16,9 @@
 """Deprecated symbols."""
 
 from tensorflow_datasets.core.deprecated import text
+from tensorflow_datasets.core.download.checksums import add_checksums_dir
 
 __all__ = [
+    'add_checksums_dir',
     'text',
 ]
