@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2021 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -109,17 +109,24 @@ class ImageFeatureTest(
                 raise_msg='are incompatible',
             ),
         ],
-        test_attributes=dict(_encoding_format=None)
+        test_attributes=dict(
+            _encoding_format=None,
+            _use_colormap=False,
+        )
     )
 
   def test_image_shaped(self):
 
-    img_shaped = randint(256, size=(32, 64, 3), dtype=np.uint8)
+    img_shaped = randint(256, size=(32, 64, 1), dtype=np.uint8)
 
     self.assertFeature(
         # Image with statically defined shape
-        feature=features_lib.Image(shape=(32, 64, 3), encoding_format='png'),
-        shape=(32, 64, 3),
+        feature=features_lib.Image(
+            shape=(32, 64, 1),
+            encoding_format='png',
+            use_colormap=True,
+        ),
+        shape=(32, 64, 1),
         dtype=tf.uint8,
         tests=[
             testing.FeatureExpectationItem(
@@ -128,12 +135,15 @@ class ImageFeatureTest(
             ),
             # 'img_shaped' shape should be static
             testing.FeatureExpectationItem(
-                value=randint(256, size=(31, 64, 3), dtype=np.uint8),
+                value=randint(256, size=(31, 64, 1), dtype=np.uint8),
                 raise_cls=ValueError,
                 raise_msg='are incompatible',
             ),
         ],
-        test_attributes=dict(_encoding_format='png')
+        test_attributes=dict(
+            _encoding_format='png',
+            _use_colormap=True,
+        )
     )
 
 
