@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2021 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,14 +13,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lint as: python3
 """Wrapper around FeatureDict to allow better control over decoding.
 """
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 from tensorflow_datasets.core.features import feature as feature_lib
 
 
@@ -28,24 +23,12 @@ class TopLevelFeature(feature_lib.FeatureConnector):
   """Top-level `FeatureConnector` to manage decoding.
 
   Note that `FeatureConnector` which are declared as `TopLevelFeature` can be
-  nested. However, only the top-level feature can be decoded.
+  nested. However, only the top-level feature should be decoded.
 
   `TopLevelFeature` allows better control over the decoding, and
   eventually better support for augmentations.
   """
 
-  def __init__(self, *args, **kwargs):
-    """Constructor."""
-    self._is_top_level = False
-    super(TopLevelFeature, self).__init__(*args, **kwargs)
-
-  def _set_top_level(self):
-    """Indicates that the feature is top level.
-
-    Internal function called by `DatasetInfo`.
-    """
-    self._is_top_level = True
-
   def decode_example(self, serialized_example, decoders=None):
     # pylint: disable=line-too-long
     """Decode the serialize examples.
@@ -61,13 +44,6 @@ class TopLevelFeature(feature_lib.FeatureConnector):
     Returns:
       example: Nested `dict` containing the decoded nested examples.
     """
-    # pylint: enable=line-too-long
-    if not self._is_top_level:
-      raise AssertionError(
-          'Feature {} can only be decoded when defined as top-level '
-          'feature, through info.features.decode_example()'.format(
-              type(self).__name__))
-
     # Step 1: Flatten the nested dict => []
     flat_example = self._flatten(serialized_example)
     flat_features = self._flatten(self)
@@ -101,30 +77,46 @@ class TopLevelFeature(feature_lib.FeatureConnector):
 
 def _decode_feature(feature, example, serialized_info, decoder):
   """Decode a single feature."""
-  # Eventually overwrite the default decoding
   if decoder is not None:
-    decoder.setup(feature=feature)
+    # If the decoder is still a dict, it means that the feature is a Dataset
+    # (it wasn't flattened).
+    if isinstance(decoder, dict):
+      decode_kwargs = dict(decoders=decoder)
+      decoder = feature
+    else:
+      # Eventually overwrite the default decoding
+      decode_kwargs = {}
+      decoder.setup(feature=feature)
   else:
+    decode_kwargs = {}
     decoder = feature
 
   sequence_rank = _get_sequence_rank(serialized_info)
   if sequence_rank == 0:
-    return decoder.decode_example(example)
+    return decoder.decode_example(example, **decode_kwargs)
   elif sequence_rank == 1:
     # Return a batch of examples from a sequence
-    return decoder.decode_batch_example(example)
+    return decoder.decode_batch_example(example, **decode_kwargs)
   elif sequence_rank > 1:
     # Use ragged tensor if the sequance rank is greater than one
-    return decoder.decode_ragged_example(example)
+    return decoder.decode_ragged_example(example, **decode_kwargs)
 
 
 def _get_sequence_rank(serialized_info):
   """Return the number of sequence dimensions of the feature."""
-  if isinstance(serialized_info, dict):
-    all_sequence_rank = [s.sequence_rank for s in serialized_info.values()]
-  else:
-    all_sequence_rank = [serialized_info.sequence_rank]
 
+  if isinstance(serialized_info, dict):
+    # If the element is a dictionary, it might correspond to a nested dataset
+    # whose serialized_info is not flattened (so it might be a nested dict).
+    all_sequence_rank = [
+        _get_sequence_rank(s) for s in serialized_info.values()
+    ]
+  else:
+    # If this is a nested dataset, we ignore the sequence_rank. We will decode
+    # the full dataset example with the Dataset decoder.
+    if serialized_info.dataset_lvl > 0:
+      return 0
+    all_sequence_rank = [serialized_info.sequence_rank]
   sequence_ranks = set(all_sequence_rank)
   if len(sequence_ranks) != 1:
     raise NotImplementedError(
