@@ -70,7 +70,9 @@ class CaltechBirds2010(tfds.core.GeneratorBasedBuilder):
             "label": tfds.features.ClassLabel(num_classes=200),
             "label_name": tfds.features.Text(),
             "bbox": tfds.features.BBoxFeature(),
-            "segmentation_mask": tfds.features.Image(shape=(None, None, 1)),
+            "segmentation_mask": tfds.features.Image(
+                shape=(None, None, 1), use_colormap=True,
+            ),
         }),
         supervised_keys=("image", "label"),
         homepage=_URL,
