@@ -68,6 +68,9 @@ variations within the category and several very similar categories.
 The dataset is divided into a training set, a validation set and a test set.
 The training set and validation set each consist of 10 images per class (totalling 1020 images each).
 The test set consists of the remaining 6149 images (minimum 20 per class).
+
+Note: The dataset by default comes with a test size larger than the train
+size. For more info see this [issue](https://github.com/tensorflow/datasets/issues/3022).
 """
 
 
