@@ -1,5 +1,5 @@
 # coding=utf-8
-# Copyright 2020 The TensorFlow Datasets Authors.
+# Copyright 2021 The TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -233,6 +233,13 @@ _TRAIN_SUBSETS = [
         sources={"es", "fr"},
         url="http://www.statmt.org/wmt13/training-parallel-un.tgz",
         path=("un/undoc.2000.{src}-en.{src}", "un/undoc.2000.{src}-en.en")),
+    SubDataset(
+        name="newscommentary_v8",
+        target="en",
+        sources={"cs", "de", "fr", "es", "ru"},
+        url="http://www.statmt.org/wmt13/training-parallel-nc-v8.tgz",
+        path=("training/news-commentary-v8.{src}-en.{src}",
+              "training/news-commentary-v8.{src}-en.en")),
     SubDataset(
         name="newscommentary_v9",
         target="en",
@@ -356,7 +363,7 @@ _TRAIN_SUBSETS = [
         url="http://www.statmt.org/wmt14/wiki-titles.tgz",
         path="wiki/hi-en/wiki-titles.hi-en"),
     SubDataset(
-        # Verified that wmt14 and wmt15 files are identical.
+        # Verified that wmt13, wmt14 and wmt15 files are identical.
         name="wikiheadlines_ru",
         target="en",
         sources={"ru"},
@@ -577,7 +584,6 @@ class WmtConfig(tfds.core.BuilderConfig):
                citation=None,
                description=None,
                language_pair=(None, None),
-               text_encoder_config=None,
                subsets=None,
                **kwargs):
     """BuilderConfig for WMT.
@@ -588,16 +594,11 @@ class WmtConfig(tfds.core.BuilderConfig):
       description: The description of the dataset.
       language_pair: pair of languages that will be used for translation. Should
                  contain 2 letter coded strings. For example: ("en", "de").
-      text_encoder_config: `tfds.deprecated.text.TextEncoderConfig` (optional),
-        configuration for the `tfds.deprecated.text.TextEncoder` used for the
-        `tfds.deprecated.text.Translation` features.
       subsets: Dict[split, list[str]]. List of the subset to use for each of the
         split. Note that WMT subclasses overwrite this parameter.
       **kwargs: keyword arguments forwarded to super.
     """
     name = "%s-%s" % (language_pair[0], language_pair[1])
-    if text_encoder_config:
-      name += "." + text_encoder_config.name
     if "name" in kwargs:  # Add name suffix for custom configs
       name += "." + kwargs.pop("name")
 
@@ -607,7 +608,6 @@ class WmtConfig(tfds.core.BuilderConfig):
     self.url = url or "http://www.statmt.org"
     self.citation = citation
     self.language_pair = language_pair
-    self.text_encoder_config = text_encoder_config
     self.subsets = subsets
 
 
@@ -659,16 +659,12 @@ class WmtTranslate(tfds.core.GeneratorBasedBuilder):
         description=_DESCRIPTION,
         features=tfds.features.Translation(
             languages=self.builder_config.language_pair,
-            encoder_config=self.builder_config.text_encoder_config),
+        ),
         supervised_keys=(src, target),
         homepage=self.builder_config.url,
         citation=self.builder_config.citation,
     )
 
-  def _vocab_text_gen(self, split_subsets, extraction_map, language):
-    for _, ex in self._generate_examples(split_subsets, extraction_map):
-      yield ex[language]
-
   def _split_generators(self, dl_manager):
     source, _ = self.builder_config.language_pair
 
@@ -706,13 +702,10 @@ class WmtTranslate(tfds.core.GeneratorBasedBuilder):
     # Extract manually downloaded files.
     manual_files = dl_manager.extract(manual_paths)
 
-    extraction_map = dict(downloaded_files, **manual_files)
+    manual_files = tf.nest.map_structure(os.fspath, manual_files)
+    downloaded_files = tf.nest.map_structure(os.fspath, downloaded_files)
 
-    # Generate vocabulary from training data if SubwordTextEncoder configured.
-    for language in self.builder_config.language_pair:
-      self.info.features[language].maybe_build_from_corpus(
-          self._vocab_text_gen(
-              self.subsets[tfds.Split.TRAIN], extraction_map, language))
+    extraction_map = dict(downloaded_files, **manual_files)
 
     return [
         tfds.core.SplitGenerator(  # pylint:disable=g-complex-comprehension
